/*
 * Copyright (c) 2004-2007 Erez Zadok
 * Copyright (c) 2004-2007 Stony Brook University
 * Copyright (c) 2004-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */

#ifndef __RES_MANAGER_H
#define __RES_MANAGER_H

#include "replay.h"
#include "trfs_pre.h"

struct context;
//Prog List
struct prog_entry_s
{
  struct replay_command* rcmdp;
  struct prog_entry_s* next;
};


int res_manager_init(char* fname,int memory_buffers_size);
int res_manager_start_trace_message(op_t op,void* cont);
int res_manager_end_trace_message(op_t op,void* cont);
int res_manager_commit();

//RAT Functions
int res_manager_lookup_rat(int val, short type);
int res_manager_append_rat(int val, short type);
inline int res_manager_get_rat_size();

//To be able to grab the dentry from the inode number - for replay_writepage()
int res_manager_resolve_dentry_from_inum(struct context* cont,int op,int pre_or_post,struct replay_command** rcmd_pre,struct replay_command** rcmd_post);
int res_manager_add_dget_op(struct replay_command** cmd,timeval_t time,long rat_loc,long thread_loc);
int res_manager_add_dput_op(struct replay_command** cmd,timeval_t time,long rat_loc,long thread_loc);
int res_manager_add_unref_dentry(long rat_loc);
int res_manager_remove_unref_dentry(long rat_loc);
int res_manager_hash_dget_op_to_thread(long rat_loc,int thread_loc);

//Optimization: For reducing memory buffers
int res_manager_add_rat_ref(struct replay_command* rcmdp,int rat_loc);

//Optimization: For reusing RAT entries - re-used by append_rat
//For Reusing Files
int res_manager_reuse_rat_entries(struct context* cont,int op,int pre_or_post);

int res_manager_init_prog();
int res_manager_insert_prog(struct prog_entry_s* prog_entry,struct replay_command* rcmdp);
struct prog_entry_s* res_manager_get_prog_list();
struct replay_command* res_manager_append_prog(struct replay_command* rcmdp);

int res_manager_append_data(int pointer, timeval_t time, char* buffer, int len,int thread_loc);


#endif //#ifndef __RES_MANAGER_H
