%{
/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 */
	/*
	 * Configuration file parser for tracefs
	 */
#include <stdio.h>
#include <errno.h>
#include "common.h"
#include "gengraph.h"

#define YYPARSE_PARAM graphset

int err;
 int yylex();
%}


%union {
  int i;
  struct block *b;
  char *c;
  struct user_output_driver *oddest;
  struct user_assembly_driver *addest;
  struct array *array;
}

%type <addest> assembly
%type <array> assembly_list
%type <oddest> output
%type <array> output_list
%type <b> rule trace_expression
%type <i> INT CUID CGID FUID FGID SID PID PGRP INO FILENAME EXT PNAME STREAM AGGREGATE OP_TYPE CKSUM BUF ENCRYPT STREAM_VERBOSITY_OP stream_verbosity_bitmap SYSCALL_ID BUFFER ASYNC COUNT SYNC_WRITES
%type <c> QSTR STR HEXSTR bitmap syscall_bitmap
%token AND OR NOT FILENAME CUID CGID QSTR EXT FUID FGID TRUE FALSE OPS HOSTNAME PORT INT HEXSTR SYSCALL_ID ASYNC COUNT
%token SYSCALLS TIME SID PID PGRP PNAME INO STR STREAM AGGREGATE OP_TYPE CKSUM COMPRESS BUF ENCRYPT STREAM_VERBOSITY_OP BUFFER SYNC_WRITES
%left OR
%left AND
%nonassoc NOT
%%

destination_list: destination destination_list | destination
        ;

destination: '{' trace_expression '{' assembly_list '}' '{' output_list '}' '}' {
	if (add_destination(graphset, $2, $4, $7) < 0) {
		YYABORT;
	}
}
| '{' '{' assembly_list '}' '{' output_list '}' '}' {
	if (add_destination(graphset, $<b>0, $3, $6) < 0) {
		YYABORT;
	}
}
;

trace_expression: trace_expression AND trace_expression { gen_and($1, $3); $$ = $3; }
| trace_expression OR trace_expression { gen_or($1, $3); $$ = $3; }
| '(' trace_expression ')' { $$ = $2; }
| NOT trace_expression { gen_not($2), $$ = $2; }
	| rule
	;

assembly_list: assembly ';' assembly_list { $$ = append_array($3, $1, sizeof(struct user_assembly_driver)); }
| assembly { $$ = make_array($1, sizeof(struct user_assembly_driver)); };

assembly: STREAM '=' '{' stream_verbosity_bitmap '}'		{ $$ = gen_assembly($1, $4); }
|	AGGREGATE	{ $$ = gen_assembly($1, 0); }
	;

output_list: output ';' output_list { $$ = append_array($3, $1, sizeof(struct user_output_driver)); }
| output { $$ = make_array($1, sizeof(struct user_output_driver)); };

output:  HOSTNAME '=' QSTR PORT '=' INT  { $$ = gen_socket_destination($3, $6, 4096); }
        | HOSTNAME '=' QSTR PORT '=' INT  BUF '=' INT { $$ = gen_socket_destination($3, $6, $9); }
	| FILENAME '=' QSTR  { $$ = gen_file_destination($3, 4096, 0); }
	| FILENAME '=' QSTR  BUF '=' INT { $$ = gen_file_destination($3, $6, 0); }
        | FILENAME '=' QSTR BUF '=' INT SYNC_WRITES '=' INT { $$ = gen_file_destination($3, $6, $9); }
	| ASYNC '=' QSTR { $$ = gen_async_destination($3, 4096, 8, 0); }
        | ASYNC '=' QSTR BUF '=' INT COUNT '=' INT SYNC_WRITES '=' INT { $$ = gen_async_destination($3, $6, $9, $12); }
	| ASYNC { $$ = gen_async_destination(NULL, 4096, 8, 0); }
        | ASYNC BUF '=' INT COUNT '=' INT { $$ = gen_async_destination(NULL, $4, $7, 0); }
	| BUFFER { $$ = gen_buffer_destination(0); }
	| BUFFER '=' INT { $$ = gen_buffer_destination($3); }
        | CKSUM '=' QSTR { $$ = gen_cksum_destination($3, 0); }
        | CKSUM '=' HEXSTR { $$ = gen_cksum_destination($3, 1); }
        | COMPRESS BUF '=' INT { $$ = gen_compress_destination($4); }
        | COMPRESS { $$ = gen_compress_destination(0); }
	| ENCRYPT '=' QSTR { $$ = gen_encrypt_destination($3, 0); }
	| ENCRYPT '=' HEXSTR { $$ = gen_encrypt_destination($3, 1); }
	;

rule: FILENAME '=' QSTR { $$ = gen_string_block($1, sdup($3)); }
	| CUID '=' INT { $$ = gen_int_block($1, $3); }
        | CGID '=' INT { $$ = gen_int_block($1, $3); }
	| EXT '=' QSTR { $$ = gen_string_block($1, sdup($3)); }
        | FUID '=' INT { $$ = gen_int_block($1, $3); }
	| FGID '=' INT  { $$ = gen_int_block($1, $3); }
	| TRUE { $$ = gen_retblk(1); }
	| FALSE { $$ = gen_retblk(0); }
	| OPS '=' '{' bitmap '}' { $$ = gen_bmap_block(TC_OP, $4); }
	| SYSCALLS '=' '{' syscall_bitmap '}' { $$ = gen_bmap_block(TC_SYSCALL, $4); }
	| TIME '=' '{' INT ',' INT '}' { $$ = $<b>0; }
	| SID '=' INT { $$ = gen_int_block($1, $3); }
	| PID '=' INT { $$ = gen_int_block($1, $3); }
	| PGRP '=' INT { $$ = gen_int_block($1, $3); }
	| PNAME '=' QSTR { $$ = gen_string_block($1, sdup($3)); }
	| INO '=' INT { $$ = gen_int_block($1, $3); }
	;


bitmap: OP_TYPE '|' bitmap { $$ = set_bit($3, $1); }
	| OP_TYPE { $$ = gen_bmap($1);  }
	;

syscall_bitmap: SYSCALL_ID '|' syscall_bitmap { $$ = set_bit($3, $1); }
	| SYSCALL_ID { $$ = gen_bmap($1); }

stream_verbosity_bitmap: STREAM_VERBOSITY_OP '|' stream_verbosity_bitmap { $$ = $3 | $1; }
	| STREAM_VERBOSITY_OP { $$ = $1; }
	;
%%

int yyerror(char *msg)
{
	fprintf(stderr, msg);
	return 0;
}

